import pyarbtools
import matplotlib.pyplot as plt
import numpy as np


def cw_pulse_test():
    fs = 100e6
    pWidth = 1e-6
    pri = 2e-6
    cf = 10e6
    data = pyarbtools.wfmBuilder.cw_pulse_generator(fs=fs, pWidth=pWidth, pri=pri, cf=cf, wfmFormat='real')

    plt.plot(data)
    plt.plot(data.real)
    plt.plot(data.imag)
    plt.show()


def export_test():
    realdata = pyarbtools.wfmBuilder.cw_pulse_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.cw_pulse_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')

    realdata = pyarbtools.wfmBuilder.sine_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.sine_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')


    realdata = pyarbtools.wfmBuilder.am_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.am_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')

    realdata = pyarbtools.wfmBuilder.cw_pulse_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.cw_pulse_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')

    realdata = pyarbtools.wfmBuilder.chirp_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.chirp_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')

    realdata = pyarbtools.wfmBuilder.barker_generator(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.barker_generator(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')

    realdata = pyarbtools.wfmBuilder.multitone(wfmFormat='real')
    iqdata = pyarbtools.wfmBuilder.multitone(wfmFormat='iq')
    pyarbtools.wfmBuilder.export_wfm(realdata, 'C:\\users\\moalliso\\Desktop\\realtest.txt')
    pyarbtools.wfmBuilder.export_wfm(iqdata, 'C:\\users\\moalliso\\Desktop\\iqtest.txt')


def convolution_test():
    fs = 100
    freq = 10
    rl = 1000
    t = np.linspace(-rl // 2, rl // 2, rl)
    sine = np.sin(2 * np.pi * freq * t)
    c = np.convolve(sine, t)

    plt.plot(sine)
    plt.plot(c)
    plt.show()


def vsg_channel_quality_correction_example():
    vsg = pyarbtools.instruments.VSG('141.121.192.207')
    vsg.configure(rfState=1, modState=1, fs=200e6, cf=1e9, amp=0)
    fileName = 'C:\\users\\moalliso\\Desktop\\mtone.csv'

    numTones = 2001
    toneSpacing = 70e3
    iq = pyarbtools.wfmBuilder.multitone(fs=vsg.fs, spacing=toneSpacing, num=numTones, phase='zero')
    pyarbtools.wfmBuilder.export_wfm(iq, fileName)

    iqCorr, h, j = pyarbtools.wfmBuilder.channel_quality_correction(iq, vsg, vsaIPAddress='127.0.0.1', vsaHardware='"Everett PXA"',
                                        cf=1e9, numTones=numTones, toneSpacing=toneSpacing, plot=True)

    vsg.download_wfm(iqCorr, 'corrected')
    vsg.play('corrected')

"""This came from examples.py"""
# def vsg_channel_quality_correction_example(ipAddress):
#     vsg = pyarbtools.instruments.VSG(ipAddress)
#     vsg.configure(rfState=1, modState=1, fs=200e6, cf=1e9, amp=0)
#
#     numTones=1025
#     toneSpacing = 40e3
#     iq = pyarbtools.wfmBuilder.multitone(fs=vsg.fs, spacing=toneSpacing, num=numTones, phase='increasing')
#
#     iqCorr = pyarbtools.wfmBuilder.channel_quality_correction(iq, vsg, vsaIPAddress='127.0.0.1', vsaHardware='"Santa Clara PXA"', cf=1e9, numTones=numTones,
#                                toneSpacing=toneSpacing)
#
#     vsg.download_wfm(iqCorr, 'corrected')
#     vsg.play('corrected')

def channel_quality_torture_test():
    vsg = pyarbtools.instruments.VSG('141.121.39.174')
    vsg.configure(rfState=1, modState=1, fs=200e6, cf=1e9, amp=0)

    # numTones = [100, 101, 200, 201, 500, 501, 1000, 1001, 2000, 2001]
    # toneSpacing = [10e3, 20e3, 50e3, 100e3, 200e3, 500e3]
    numTones = [1000]
    toneSpacing = [10e3]
    tones = []
    spacing = []
    mag = []
    phase = []

    for n in numTones:
        for s in toneSpacing:
            try:
                iq = pyarbtools.wfmBuilder.multitone(fs=vsg.fs, spacing=s, num=n, phase='zero')
                a, m, p = pyarbtools.wfmBuilder.channel_quality_correction(iq, vsg, vsaIPAddress='127.0.0.1', vsaHardware='"Santa Clara PXA"', cf=1e9, numTones=n, toneSpacing=s)
                tones.append(n)
                spacing.append(s)
                mag.append(m)
                phase.append(p)
            except pyarbtools.error.WfmBuilderError as e:
                print(str(e))

    with open('C:\\users\\moalliso\\Desktop\\correctiontesting.txt', 'w+') as f:
        for t, s, m, p in zip(tones, spacing, mag, phase):
            f.write(f'\nFor {t} tones at {s / 1000} kHz spacing:\nMag: {m} %\nPhase: {p}%')


def recording_test():
    vsa = pyarbtools.communications.SocketInstrument('127.0.0.1', port=5025)

    fs = 200e6
    numTones = 1001
    toneSpacing = 40e3
    path = 'C:\\users\\moalliso\\desktop\\'
    phase = 'increasing'

    iqData = pyarbtools.wfmBuilder.multitone(fs=fs, spacing=toneSpacing, num=numTones, phase=phase)
    wfm = pyarbtools.wfmBuilder.WFM(iqData, wfmFormat='iq', fs=fs, wfmID=f'mtone_{phase}')
    wfm.repeat(10)
    wfm.export(path=path, vsaCompatible=True)

    vsa.write('input:data rec')
    vsa.write(f'input:recording:file {wfm.fileName}')
    # vsa.write('init:cont 1')
    # vsa.write('init:immed')


def main():
    # cw_pulse_test()
    # export_test()
    # convolution_test()
    # channel_quality_torture_test()
    vsg_channel_quality_correction_example()

if __name__ == '__main__':
    main()
